"""
A demonstration of the Mapper.

Author: Per Erik Strandberg, 2017

The source code in this bundle explores the subgraph isomorphism problem
Copyright (C) 2018 P. E. Strandberg, Westermo Research and Development AB

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""

from mapper import Mapper
from network import Graph
from testdata import TestGraphs

print "\n\n# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #"
print "#"
print "# 1 - create graphs"
print "#"

TG = TestGraphs()
G = TG.graphs['G0']

H = Graph()

ringmembers = ['C1', 'RS', 'S', 'RN']
for member in ringmembers:
    H.add_node(member)

for i in range(len(ringmembers)):
    H.add_edge(ringmembers[i], ringmembers[i-1])

H.add_node('C2')
H.add_edge('C2', 'S')

print "Created two graphs."
print "  G:",
print G
print "  H:",
print H

print "\n\n# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #"
print "#"
print "# 2 - run mapper"
print "#"

M1 = Mapper(G, H, find_all=False)

(hmap, gmap, mapgraph) = M1.solutions[0]

print "\n\n# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #"
print "#"
print "# 3 - run mapper again and use another search strategy"
print "#"

old_ones = list()
old_ones.append(tuple(M1.solutions[-1]))

M2 = Mapper(G, H, find_all=False, check_old=True, old_ones=old_ones)


print "\n\n# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #"
print "#"
print "# 4 - compare results"
print "#"

(hmap1, gmap1, _) = M1.solutions[0]
(hmap2, gmap2, _) = M2.solutions[0]

print "In the first mapping:"
for idx in range(len(hmap1)):
    print "  H-node %s was mapped onto G-node %s" % (hmap1[idx], gmap1[idx])

print "In the second mapping:"
for idx in range(len(hmap2)):
    print "  H-node %s was mapped onto G-node %s" % (hmap2[idx], gmap2[idx])
