"""
Another demonstration of the Mapper.

Author: Per Erik Strandberg, 2017

The source code in this bundle explores the subgraph isomorphism problem
Copyright (C) 2018 P. E. Strandberg, Westermo Research and Development AB

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""

from mapper import Mapper
from network import Graph

# G is a cycle of 5
G = Graph()
G.add_node(41)
G.add_node(42)
G.add_node(43)
G.add_node(44)
G.add_node(45)

G.add_edge(41, 42)
G.add_edge(42, 43)
G.add_edge(43, 44)
G.add_edge(44, 45)
G.add_edge(45, 41)

# H is a bus of 3
H = Graph()
H.add_node("A")
H.add_node("B")
H.add_node("C")

H.add_edge("A", "B")
H.add_edge("B", "C")

# Map once
M1 = Mapper(G, H, find_all=False)
(hmap, gmap, _) = M1.solutions[-1]

# check which nodes in G are used
g_first = gmap

print ""
print "-" * 40
print ""
print "In the first mapping, we used: %s" % sorted(g_first)
print ""
print "-" * 40
print ""


# Map some more times
old_ones = list()
old_ones.append(tuple(M1.solutions[-1]))

iterations = 7

while 0 < iterations:
    iterations -= 1
    M = Mapper(G, H, find_all=False, check_old=True, old_ones=old_ones)
    old_ones.append(tuple(M.solutions[-1]))

# check how often nodes in G were used
g_used = dict()

for old_one in old_ones:
    (_, gmap, _) = old_one

    for node in gmap:
        if node not in g_used:
            g_used[node] = 0
        g_used[node] += 1

# print the result


print ""
print "-" * 40
print ""
print "In total we used the nodes in G this many times:"
for node in G.get_nodes():
    print "  node %s was used %s times" % (node, g_used[node])
